@@ -1,61 +1,34 @@
-#[cfg(test)]
-use {
-    crate::ErrorType
-};
-
-#[cfg(test)]
-fn _eval(src: &'static [u8], expected: &'static [u8], cfg: &super::Cfg) -> () {
+fn eval_with_cfg(src: &'static [u8], expected: &'static [u8], cfg: &super::Cfg) {
     let mut code = src.to_vec();
-    match super::with_friendly_error(&mut code, cfg) {
-        Ok(len) => {
-            assert_eq!(std::str::from_utf8(&code[..len]).unwrap(), std::str::from_utf8(expected).unwrap());
-        }
-        Err(super::FriendlyError { code_context, message, .. }) => {
-            println!("{}", message);
-            println!("{}", code_context);
-            assert!(false);
-        }
-    };
+    let min = super::minify(&mut code, cfg);
+    assert_eq!(
+        std::str::from_utf8(&min).unwrap(),
+        std::str::from_utf8(expected).unwrap(),
+    );
 }
 
-#[cfg(test)]
-fn _eval_error(src: &'static [u8], expected: ErrorType, cfg: &super::Cfg) -> () {
-    let mut code = src.to_vec();
-    assert_eq!(super::in_place(&mut code, cfg).unwrap_err().error_type, expected);
+fn eval(src: &'static [u8], expected: &'static [u8]) {
+    eval_with_cfg(src, expected, &super::Cfg::new());
 }
 
-#[cfg(test)]
-fn eval(src: &'static [u8], expected: &'static [u8]) -> () {
-    _eval(src, expected, &super::Cfg {
-        minify_js: false,
-        minify_css: false,
-    });
+fn eval_with_keep_html_head(src: &'static [u8], expected: &'static [u8]) -> () {
+    let mut cfg = super::Cfg::new();
+    cfg.keep_html_and_head_opening_tags = true;
+    eval_with_cfg(src, expected, &cfg);
 }
 
-#[cfg(test)]
-fn eval_error(src: &'static [u8], expected: ErrorType) -> () {
-    _eval_error(src, expected, &super::Cfg {
-        minify_js: false,
-        minify_css: false,
-    });
-}
-
-#[cfg(test)]
 #[cfg(feature = "js-esbuild")]
 fn eval_with_js_min(src: &'static [u8], expected: &'static [u8]) -> () {
-    _eval(src, expected, &super::Cfg {
-        minify_js: true,
-        minify_css: false,
-    });
+    let mut cfg = super::Cfg::new();
+    cfg.minify_js = true;
+    eval_with_cfg(src, expected, &cfg);
 }
 
-#[cfg(test)]
 #[cfg(feature = "js-esbuild")]
 fn eval_with_css_min(src: &'static [u8], expected: &'static [u8]) -> () {
-    _eval(src, expected, &super::Cfg {
-        minify_js: false,
-        minify_css: true,
-    });
+    let mut cfg = super::Cfg::new();
+    cfg.minify_css = true;
+    eval_with_cfg(src, expected, &cfg);
 }
 
 #[test]
@@ -80,7 +53,10 @@ fn test_collapse_destroy_whole_and_trim_whitespace() {
     eval(b"<ul>   \n&#32;   </ul>", b"<ul></ul>");
     eval(b"<ul>   \n&#32;a   </ul>", b"<ul>a</ul>");
     eval(b"<ul>   \n&#32;a   b   </ul>", b"<ul>a b</ul>");
-    eval(b"<ul>   \n&#32;a<pre></pre>   <pre></pre>b   </ul>", b"<ul>a<pre></pre><pre></pre>b</ul>");
+    eval(
+        b"<ul>   \n&#32;a<pre></pre>   <pre></pre>b   </ul>",
+        b"<ul>a<pre></pre><pre></pre>b</ul>",
+    );
     // Tag names should be case insensitive.
     eval(b"<uL>   \n&#32;a   b   </UL>", b"<ul>a b</ul>");
 }
@@ -88,33 +64,70 @@ fn test_collapse_destroy_whole_and_trim_whitespace() {
 #[test]
 fn test_no_whitespace_minification() {
     eval(b"<pre>   \n&#32; \t   </pre>", b"<pre>   \n  \t   </pre>");
-    eval(b"<textarea>   \n&#32; \t   </textarea>", b"<textarea>   \n  \t   </textarea>");
+    eval(
+        b"<textarea>   \n&#32; \t   </textarea>",
+        b"<textarea>   \n  \t   </textarea>",
+    );
     // Tag names should be case insensitive.
     eval(b"<pRe>   \n&#32; \t   </PRE>", b"<pre>   \n  \t   </pre>");
-    eval(b"<pre>  <span>  1    2   </span>  </pre>", b"<pre>  <span>  1    2   </span>  </pre>");
-    eval(b"<pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre>", b"<pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre>");
-    eval(b"<div>  <pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre>  </div>", b"<div><pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre></div>");
-    eval(br#"<pre><code>fn main() {
+    eval(
+        b"<pre>  <span>  1    2   </span>  </pre>",
+        b"<pre>  <span>  1    2   </span>  </pre>",
+    );
+    eval(
+        b"<pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre>",
+        b"<pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre>",
+    );
+    eval(
+        b"<div>  <pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre>  </div>",
+        b"<div><pre>  <span>  1 <pre>\n</pre>    2   </span>  </pre></div>",
+    );
+    eval(
+        br#"<pre><code>fn main() {
   println!("Hello, world!");
   <span>loop {
     println!("Hello, world!");
   }</span>
 }
-</code></pre>"#, br#"<pre><code>fn main() {
+</code></pre>"#,
+        br#"<pre><code>fn main() {
   println!("Hello, world!");
   <span>loop {
     println!("Hello, world!");
   }</span>
 }
-</code></pre>"#);
+</code></pre>"#,
+    );
+}
+
+#[test]
+fn test_parsing_extra_head_tag() {
+    // Extra `<head>` in `<label>` should be dropped, so whitespace around `<head>` should be joined and therefore trimmed due to `<label>` whitespace rules.
+    eval_with_keep_html_head(
+        b"<html><head><meta><head><link><head><body><label>  <pre> </pre> <head>  </label>",
+        b"<html><head><meta><link><body><label><pre> </pre></label>",
+    );
+    // Same as above except it's a `</head>`, which should get reinterpreted as a `<head>`.
+    eval_with_keep_html_head(
+        b"<html><head><meta><head><link><head><body><label>  <pre> </pre> </head>  </label>",
+        b"<html><head><meta><link><body><label><pre> </pre></label>",
+    );
+    // `<head>` gets implicitly closed by `<body>`, so any following `</head>` should be ignored. (They should be anyway, since `</head>` would not be a valid closing tag.)
+    eval_with_keep_html_head(
+        b"<html><head><body><label> </head> </label>",
+        b"<html><head><body><label></label>",
+    );
 }
 
 #[test]
 fn test_parsing_omitted_closing_tag() {
-    eval(b"<html>", b"<html>");
-    eval(b" <html>\n", b"<html>");
-    eval(b" <!doctype html> <html>\n", b"<!doctype html><html>");
-    eval(b"<!doctype html><html><div> <p>Foo</div></html>", b"<!doctype html><html><div><p>Foo</div>");
+    eval_with_keep_html_head(b"<html>", b"<html>");
+    eval_with_keep_html_head(b" <html>\n", b"<html>");
+    eval_with_keep_html_head(b" <!doctype html> <html>\n", b"<!doctype html><html>");
+    eval_with_keep_html_head(
+        b"<!doctype html><html><div> <p>Foo</div></html>",
+        b"<!doctype html><html><div><p>Foo</div>",
+    );
 }
 
 #[test]
@@ -129,33 +142,57 @@ fn test_self_closing_svg_tag_whitespace_removal() {
 
 #[test]
 fn test_parsing_with_omitted_tags() {
-    eval(b"<ul><li>1<li>2<li>3</ul>", b"<ul><li>1<li>2<li>3</ul>");
-    eval(b"<rt>", b"<rt>");
-    eval(b"<rt><rp>1</rp><div></div>", b"<rt><rp>1</rp><div></div>");
-    eval(b"<div><rt></div>", b"<div><rt></div>");
-    eval(b"<html><head><body>", b"<html><head><body>");
-    eval(b"<html><head><body>", b"<html><head><body>");
+    eval_with_keep_html_head(b"<ul><li>1<li>2<li>3</ul>", b"<ul><li>1<li>2<li>3</ul>");
+    eval_with_keep_html_head(b"<rt>", b"<rt>");
+    eval_with_keep_html_head(b"<rt><rp>1</rp><div></div>", b"<rt><rp>1</rp><div></div>");
+    eval_with_keep_html_head(b"<div><rt></div>", b"<div><rt></div>");
+    eval_with_keep_html_head(b"<html><head><body>", b"<html><head><body>");
+    eval_with_keep_html_head(b"<html><head><body>", b"<html><head><body>");
     // Tag names should be case insensitive.
-    eval(b"<rt>", b"<rt>");
+    eval_with_keep_html_head(b"<rt>", b"<rt>");
 }
 
 #[test]
 fn test_unmatched_closing_tag() {
-    eval_error(b"Hello</p>Goodbye", ErrorType::UnexpectedClosingTag);
-    eval_error(b"Hello<br></br>Goodbye", ErrorType::UnexpectedClosingTag);
-    eval_error(b"<div>Hello</p>Goodbye", ErrorType::ClosingTagMismatch { expected: "div".to_string(), got: "p".to_string() });
-    eval_error(b"<ul><li>a</p>", ErrorType::ClosingTagMismatch { expected: "ul".to_string(), got: "p".to_string() });
-    eval_error(b"<ul><li><rt>a</p>", ErrorType::ClosingTagMismatch { expected: "ul".to_string(), got: "p".to_string() });
-    eval_error(b"<html><head><body><ul><li><rt>a</p>", ErrorType::ClosingTagMismatch { expected: "ul".to_string(), got: "p".to_string() });
+    eval_with_keep_html_head(b"Hello</p>Goodbye", b"Hello<p>Goodbye");
+    eval_with_keep_html_head(b"Hello<br></br>Goodbye", b"Hello<br>Goodbye");
+    eval_with_keep_html_head(b"<div>Hello</p>Goodbye", b"<div>Hello<p>Goodbye");
+    eval_with_keep_html_head(b"<ul><li>a</p>", b"<ul><li>a<p>");
+    eval_with_keep_html_head(b"<ul><li><rt>a</p>", b"<ul><li><rt>a<p>");
+    eval_with_keep_html_head(
+        b"<html><head><body><ul><li><rt>a</p>",
+        b"<html><head><body><ul><li><rt>a<p>",
+    );
+}
+
+#[test]
+fn test_removal_of_html_and_head_opening_tags() {
+    // Even though `<head>` is dropped, it's still parsed, so its content is still subject to `<head>` whitespace minification rules.
+    eval(
+        b"<!DOCTYPE html><html><head>  <meta> <body>",
+        b"<!DOCTYPE html><meta><body>",
+    );
+    // The tag should not be dropped if it has attributes.
+    eval(
+        b"<!DOCTYPE html><html lang=en><head>  <meta> <body>",
+        b"<!DOCTYPE html><html lang=en><meta><body>",
+    );
 }
 
 #[test]
 fn test_removal_of_optional_tags() {
-    eval(b"<ul><li>1</li><li>2</li><li>3</li></ul>", b"<ul><li>1<li>2<li>3</ul>");
-    eval(b"<rt></rt>", b"<rt>");
-    eval(b"<rt></rt><rp>1</rp><div></div>", b"<rt><rp>1</rp><div></div>");
-    eval(b"<div><rt></rt></div>", b"<div><rt></div>");
-    eval(br#"
+    eval_with_keep_html_head(
+        b"<ul><li>1</li><li>2</li><li>3</li></ul>",
+        b"<ul><li>1<li>2<li>3</ul>",
+    );
+    eval_with_keep_html_head(b"<rt></rt>", b"<rt>");
+    eval_with_keep_html_head(
+        b"<rt></rt><rp>1</rp><div></div>",
+        b"<rt><rp>1</rp><div></div>",
+    );
+    eval_with_keep_html_head(b"<div><rt></rt></div>", b"<div><rt></div>");
+    eval_with_keep_html_head(
+        br#"
         <html>
             <head>
             </head>
@@ -163,9 +200,11 @@ fn test_removal_of_optional_tags() {
             <body>
             </body>
         </html>
-    "#, b"<html><head><body>");
+    "#,
+        b"<html><head><body>",
+    );
     // Tag names should be case insensitive.
-    eval(b"<RT></rt>", b"<rt>");
+    eval_with_keep_html_head(b"<RT></rt>", b"<rt>");
 }
 
 #[test]
@@ -173,7 +212,10 @@ fn test_removal_of_optional_closing_p_tag() {
     eval(b"<p></p><address></address>", b"<p><address></address>");
     eval(b"<p></p>", b"<p>");
     eval(b"<map><p></p></map>", b"<map><p></p></map>");
-    eval(b"<map><p></p><address></address></map>", b"<map><p><address></address></map>");
+    eval(
+        b"<map><p></p><address></address></map>",
+        b"<map><p><address></address></map>",
+    );
 }
 
 #[test]
@@ -191,7 +233,10 @@ fn test_attr_single_quoted_value_minification() {
     eval(b"<a b=\"&quot;hello\"></a>", b"<a b='\"hello'></a>");
     eval(b"<a b='\"hello'></a>", b"<a b='\"hello'></a>");
     eval(b"<a b='/>a'></a>", b"<a b=\"/>a\"></a>");
-    eval(b"<a b=&#x20;he&quot;llo&#x20;></a>", b"<a b=' he\"llo '></a>");
+    eval(
+        b"<a b=&#x20;he&quot;llo&#x20;></a>",
+        b"<a b=' he\"llo '></a>",
+    );
 }
 
 #[test]
@@ -208,7 +253,10 @@ fn test_attr_unquoted_value_minification() {
 #[test]
 fn test_class_attr_value_minification() {
     eval(b"<a class=&#x20;c></a>", b"<a class=c></a>");
-    eval(b"<a class=&#x20;c&#x20&#x20;d&#x20></a>", b"<a class=\"c d\"></a>");
+    eval(
+        b"<a class=&#x20;c&#x20&#x20;d&#x20></a>",
+        b"<a class=\"c d\"></a>",
+    );
     eval(b"<a class=&#x20&#x20&#x20;&#x20></a>", b"<a></a>");
     eval(b"<a class=\"  c\n \n  \"></a>", b"<a class=c></a>");
     eval(b"<a class=\"  c\n \nd  \"></a>", b"<a class=\"c d\"></a>");
@@ -223,13 +271,34 @@ fn test_class_attr_value_minification() {
 #[test]
 fn test_d_attr_value_minification() {
     eval(b"<svg><path d=&#x20;c /></svg>", b"<svg><path d=c /></svg>");
-    eval(b"<svg><path d=&#x20;c&#x20&#x20;d&#x20 /></svg>", b"<svg><path d=\"c d\"/></svg>");
-    eval(b"<svg><path d=&#x20;&#x20&#x20&#x20 /></svg>", b"<svg><path/></svg>");
-    eval(b"<svg><path d=\"  c\n \n  \" /></svg>", b"<svg><path d=c /></svg>");
-    eval(b"<svg><path d=\"  c\n \nd  \" /></svg>", b"<svg><path d=\"c d\"/></svg>");
-    eval(b"<svg><path d=\"  \n \n  \" /></svg>", b"<svg><path/></svg>");
-    eval(b"<svg><path d='  c\n \n  ' /></svg>", b"<svg><path d=c /></svg>");
-    eval(b"<svg><path d='  c\n \nd  ' /></svg>", b"<svg><path d=\"c d\"/></svg>");
+    eval(
+        b"<svg><path d=&#x20;c&#x20&#x20;d&#x20 /></svg>",
+        b"<svg><path d=\"c d\"/></svg>",
+    );
+    eval(
+        b"<svg><path d=&#x20;&#x20&#x20&#x20 /></svg>",
+        b"<svg><path/></svg>",
+    );
+    eval(
+        b"<svg><path d=\"  c\n \n  \" /></svg>",
+        b"<svg><path d=c /></svg>",
+    );
+    eval(
+        b"<svg><path d=\"  c\n \nd  \" /></svg>",
+        b"<svg><path d=\"c d\"/></svg>",
+    );
+    eval(
+        b"<svg><path d=\"  \n \n  \" /></svg>",
+        b"<svg><path/></svg>",
+    );
+    eval(
+        b"<svg><path d='  c\n \n  ' /></svg>",
+        b"<svg><path d=c /></svg>",
+    );
+    eval(
+        b"<svg><path d='  c\n \nd  ' /></svg>",
+        b"<svg><path d=\"c d\"/></svg>",
+    );
     eval(b"<svg><path d='  \n \n  ' /></svg>", b"<svg><path/></svg>");
     // Attribute names should be case insensitive.
     eval(b"<svg><path D='  \n \n  ' /></svg>", b"<svg><path/></svg>");
@@ -268,12 +337,27 @@ fn test_default_attr_value_removal() {
 
 #[test]
 fn test_script_type_attr_value_removal() {
-    eval(b"<script type=\"application/ecmascript\"></script>", b"<script></script>");
-    eval(b"<script type=\"application/javascript\"></script>", b"<script></script>");
-    eval(b"<script type=\"text/jscript\"></script>", b"<script></script>");
-    eval(b"<script type=\"text/plain\"></script>", b"<script type=text/plain></script>");
+    eval(
+        b"<script type=\"application/ecmascript\"></script>",
+        b"<script></script>",
+    );
+    eval(
+        b"<script type=\"application/javascript\"></script>",
+        b"<script></script>",
+    );
+    eval(
+        b"<script type=\"text/jscript\"></script>",
+        b"<script></script>",
+    );
+    eval(
+        b"<script type=\"text/plain\"></script>",
+        b"<script type=text/plain></script>",
+    );
     // Tag and attribute names should be case insensitive.
-    eval(b"<SCRipt TYPE=\"application/ecmascript\"></SCrIPT>", b"<script></script>");
+    eval(
+        b"<SCRipt TYPE=\"application/ecmascript\"></SCrIPT>",
+        b"<script></script>",
+    );
 }
 
 #[test]
@@ -287,9 +371,15 @@ fn test_empty_attr_value_removal() {
 
 #[test]
 fn test_space_between_attrs_minification() {
-    eval(b"<div a=\" \" b=\" \"></div>", b"<div a=\" \"b=\" \"></div>");
+    eval(
+        b"<div a=\" \" b=\" \"></div>",
+        b"<div a=\" \"b=\" \"></div>",
+    );
     eval(b"<div a=' ' b=\" \"></div>", b"<div a=\" \"b=\" \"></div>");
-    eval(b"<div a=&#x20 b=\" \"></div>", b"<div a=\" \"b=\" \"></div>");
+    eval(
+        b"<div a=&#x20 b=\" \"></div>",
+        b"<div a=\" \"b=\" \"></div>",
+    );
     eval(b"<div a=\"1\" b=\" \"></div>", b"<div a=1 b=\" \"></div>");
     eval(b"<div a='1' b=\" \"></div>", b"<div a=1 b=\" \"></div>");
     eval(b"<div a=\"a\"b=\"b\"></div>", b"<div a=a b=b></div>");
@@ -309,7 +399,10 @@ fn test_hexadecimal_entity_decoding() {
     eval(b"&#x000000000000000000000000000000000000000000030;", b"0");
     eval(b"&#x1151;", b"\xe1\x85\x91");
     eval(b"&#x11FFFF;", b"\xef\xbf\xbd");
-    eval(b"&#xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF;", b"\xef\xbf\xbd");
+    eval(
+        b"&#xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF;",
+        b"\xef\xbf\xbd",
+    );
 }
 
 #[test]
@@ -322,7 +415,10 @@ fn test_decimal_entity_decoding() {
     eval(b"&#000000000000000000000000000000000000000000048;", b"0");
     eval(b"&#4433;", b"\xe1\x85\x91");
     eval(b"&#1114112;", b"\xef\xbf\xbd");
-    eval(b"&#999999999999999999999999999999999999999999999;", b"\xef\xbf\xbd");
+    eval(
+        b"&#999999999999999999999999999999999999999999999;",
+        b"\xef\xbf\xbd",
+    );
 }
 
 #[test]
@@ -342,9 +438,18 @@ fn test_named_entity_decoding() {
 
     // Named entities not ending with ';' in attr values are not decoded if immediately
     // followed by an alphanumeric or `=` character. (See parser for more details.)
-    eval(br#"<a href="exam ple?&gta=5"></a>"#, br#"<a href="exam ple?&gta=5"></a>"#);
-    eval(br#"<a href="exam ple?&gt=5"></a>"#, br#"<a href="exam ple?&gt=5"></a>"#);
-    eval(br#"<a href="exam ple?&gt~5"></a>"#, br#"<a href="exam ple?>~5"></a>"#);
+    eval(
+        br#"<a href="exam ple?&gta=5"></a>"#,
+        br#"<a href="exam ple?&gta=5"></a>"#,
+    );
+    eval(
+        br#"<a href="exam ple?&gt=5"></a>"#,
+        br#"<a href="exam ple?&gt=5"></a>"#,
+    );
+    eval(
+        br#"<a href="exam ple?&gt~5"></a>"#,
+        br#"<a href="exam ple?>~5"></a>"#,
+    );
 }
 
 #[test]
@@ -424,9 +529,15 @@ fn test_left_chevron_in_content() {
 
 #[test]
 fn test_comments_removal() {
-    eval(b"<pre>a <!-- akd--sj\n <!-- \t\0f--ajk--df->lafj -->  b</pre>", b"<pre>a   b</pre>");
+    eval(
+        b"<pre>a <!-- akd--sj\n <!-- \t\0f--ajk--df->lafj -->  b</pre>",
+        b"<pre>a   b</pre>",
+    );
     eval(b"&a<!-- akd--sj\n <!-- \t\0f--ajk--df->lafj -->mp", b"&amp");
-    eval(b"<script><!-- akd--sj\n <!-- \t\0f--ajk--df->lafj --></script>", b"<script><!-- akd--sj\n <!-- \t\0f--ajk--df->lafj --></script>");
+    eval(
+        b"<script><!-- akd--sj\n <!-- \t\0f--ajk--df->lafj --></script>",
+        b"<script><!-- akd--sj\n <!-- \t\0f--ajk--df->lafj --></script>",
+    );
 }
 
 #[test]
@@ -439,30 +550,62 @@ fn test_processing_instructions() {
 #[test]
 fn test_js_minification() {
     eval_with_js_min(b"<script>let a = 1;</script>", b"<script>let a=1;</script>");
-    eval_with_js_min(br#"
+    eval_with_js_min(
+        br#"
         <script>let a = 1;</script>
         <script>let b = 2;</script>
-    "#, b"<script>let a=1;</script><script>let b=2;</script>");
-    eval_with_js_min(b"<scRIPt type=text/plain>   alert(1.00000);   </scripT>", b"<script type=text/plain>   alert(1.00000);   </script>");
-    eval_with_js_min(br#"
+    "#,
+        b"<script>let a=1;</script><script>let b=2;</script>",
+    );
+    eval_with_js_min(
+        b"<scRIPt type=text/plain>   alert(1.00000);   </scripT>",
+        b"<script type=text/plain>   alert(1.00000);   </script>",
+    );
+    eval_with_js_min(
+        br#"
         <script>
             // This is a comment.
             let a = 1;
         </script>
-    "#, b"<script>let a=1;</script>");
+    "#,
+        b"<script>let a=1;</script>",
+    );
 }
 
 #[cfg(feature = "js-esbuild")]
 #[test]
 fn test_js_minification_unintentional_closing_tag() {
-    eval_with_js_min(br#"<script>let a = "</" + "script>";</script>"#, br#"<script>let a="<\/script>";</script>"#);
-    eval_with_js_min(br#"<script>let a = "</S" + "cRiPT>";</script>"#, br#"<script>let a="<\/ScRiPT>";</script>"#);
-    eval_with_js_min(br#"<script>let a = "\u003c/script>";</script>"#, br#"<script>let a="<\/script>";</script>"#);
-    eval_with_js_min(br#"<script>let a = "\u003c/scrIPt>";</script>"#, br#"<script>let a="<\/scrIPt>";</script>"#);
+    eval_with_js_min(
+        br#"<script>let a = "</" + "script>";</script>"#,
+        br#"<script>let a="<\/script>";</script>"#,
+    );
+    eval_with_js_min(
+        br#"<script>let a = "</S" + "cRiPT>";</script>"#,
+        br#"<script>let a="<\/ScRiPT>";</script>"#,
+    );
+    eval_with_js_min(
+        br#"<script>let a = "\u003c/script>";</script>"#,
+        br#"<script>let a="<\/script>";</script>"#,
+    );
+    eval_with_js_min(
+        br#"<script>let a = "\u003c/scrIPt>";</script>"#,
+        br#"<script>let a="<\/scrIPt>";</script>"#,
+    );
 }
 
 #[cfg(feature = "js-esbuild")]
 #[test]
 fn test_css_minification() {
-    eval_with_css_min(b"<style>div { color: yellow }</style>", b"<style>div{color:#ff0}</style>");
+    // `<style>` contents.
+    eval_with_css_min(
+        b"<style>div { color: yellow }</style>",
+        b"<style>div{color:#ff0}</style>",
+    );
+    // `style` attributes.
+    eval_with_css_min(
+        br#"<div style="color: yellow;"></div>"#,
+        br#"<div style=color:#ff0></div>"#,
+    );
+    // `style` attributes are removed if fully minified away.
+    eval_with_css_min(br#"<div style="  /*  */   "></div>"#, br#"<div></div>"#);
 }
